@@ -20,9 +20,9 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#include "online-accounts-ui/globals.h"
 #include "onlineaccountsui_interface.h"
 #include "setup.h"
-#include "src/globals.h"
 
 #include <QDBusConnection>
 #include <QDBusPendingCallWatcher>
@@ -30,9 +30,10 @@
 #include <QGuiApplication>
 #include <QWindow>
 #include <climits>
+#include <unistd.h>
 
 using namespace OnlineAccountsClient;
-using namespace com::canonical;
+using namespace com::ubuntu;
 
 namespace OnlineAccountsClient {
 
@@ -53,6 +54,7 @@ private Q_SLOTS:
 
 private:
     OnlineAccountsUi m_onlineAccountsUi;
+    QString m_applicationId;
     QString m_serviceTypeId;
     QString m_providerId;
     mutable Setup *q_ptr;
@@ -74,9 +76,23 @@ void SetupPrivate::exec()
 {
     QVariantMap options;
 
+    options.insert(OAU_KEY_PID, uint(getpid()));
+
     QWindow *window = clientWindow();
     if (window) {
         options.insert(OAU_KEY_WINDOW_ID, window->winId());
+        /* TODO: remove this hack once Mir supports window reparenting.
+         * Since Mir always return the same window Id for all windows, we use
+         * the process ID instead; for the time being this is acceptable since
+         * Mir/unity8 don't support more than one window per process.
+         * See: https://bugs.launchpad.net/bugs/1153666 */
+        if (QGuiApplication::platformName().startsWith("ubuntu")) {
+            options.insert(OAU_KEY_WINDOW_ID, uint(getpid()));
+        }
+    }
+
+    if (!m_applicationId.isEmpty()) {
+        options.insert(OAU_KEY_APPLICATION, m_applicationId);
     }
 
     if (!m_serviceTypeId.isEmpty()) {
@@ -113,14 +129,18 @@ void SetupPrivate::onRequestAccessReply(QDBusPendingCallWatcher *watcher)
 
     watcher->deleteLater();
 
+    QVariantMap response;
+
     QDBusPendingReply<QVariantMap> reply = *watcher;
     if (reply.isError()) {
         qWarning() << "RequestAccess failed:" << reply.error();
+        response.insert("errorName", reply.error().name());
     } else {
         // At the moment, we don't have any use for the reply.
+        response = reply.value();
     }
 
-    Q_EMIT q->finished();
+    Q_EMIT q->finished(response);
 }
 
 /*!
@@ -155,6 +175,7 @@ void SetupPrivate::onRequestAccessReply(QDBusPendingCallWatcher *watcher)
  *
  *     Setup {
  *         id: setup
+ *         applicationId: "MyApp"
  *         providerId: "facebook"
  *     }
  * }
@@ -171,6 +192,27 @@ Setup::~Setup()
     delete d_ptr;
 }
 
+/*!
+ * \qmlproperty string Setup::applicationId
+ * Specifies which application is asking for access. The value of this string
+ * must be equal to the filename of the XML application file (installed in \c
+ * /usr/share/accounts/applications/ or \c
+ * ~/.local/share/accounts/applications/) minus the \c .application suffix.
+ */
+void Setup::setApplicationId(const QString &applicationId)
+{
+    Q_D(Setup);
+    if (applicationId == d->m_applicationId) return;
+    d->m_applicationId = applicationId;
+    Q_EMIT applicationIdChanged();
+}
+
+QString Setup::applicationId() const
+{
+    Q_D(const Setup);
+    return d->m_applicationId;
+}
+
 /*!
  * \qmlproperty string Setup::serviceTypeId
  * If set to a valid service type, the user will be asked to create an Online
