@@ -24,7 +24,13 @@
 #include "request.h"
 #include "ui-proxy.h"
 
+#include <Accounts/Manager>
+#include <Accounts/Provider>
+#include <QByteArray>
+#include <QDataStream>
 #include <QDir>
+#include <QDomDocument>
+#include <QDomElement>
 #include <QFileInfo>
 #include <QLocalServer>
 #include <QLocalSocket>
@@ -49,20 +55,25 @@ public:
     inline UiProxyPrivate(pid_t clientPid, UiProxy *pluginProxy);
     inline ~UiProxyPrivate();
 
+    void setStatus(UiProxy::Status status);
     bool setupSocket();
     bool init();
     void sendOperation(const QVariantMap &data);
     void sendRequest(int requestId, Request *request);
-    void setupPromptSession();
+    bool setupPromptSession();
+    QString findAppArmorProfile();
+    void startProcess();
 
 private Q_SLOTS:
     void onNewConnection();
+    void onDisconnected();
     void onDataReady(QByteArray &data);
     void onRequestCompleted();
     void onFinishedTimer();
 
 private:
     QProcess m_process;
+    UiProxy::Status m_status;
     QLocalServer m_server;
     QLocalSocket *m_socket;
     OnlineAccountsUi::Ipc m_ipc;
@@ -71,7 +82,9 @@ private:
     QMap<int,Request*> m_requests;
     QStringList m_handlers;
     pid_t m_clientPid;
+    QString m_providerId;
     PromptSessionP m_promptSession;
+    QStringList m_arguments;
     mutable UiProxy *q_ptr;
 };
 
@@ -79,6 +92,7 @@ private:
 
 UiProxyPrivate::UiProxyPrivate(pid_t clientPid, UiProxy *uiProxy):
     QObject(uiProxy),
+    m_status(UiProxy::Null),
     m_socket(0),
     m_nextRequestId(0),
     m_clientPid(clientPid),
@@ -109,6 +123,14 @@ UiProxyPrivate::~UiProxyPrivate()
     m_server.close();
 }
 
+void UiProxyPrivate::setStatus(UiProxy::Status status)
+{
+    Q_Q(UiProxy);
+    if (m_status == status) return;
+    m_status = status;
+    Q_EMIT q->statusChanged();
+}
+
 void UiProxyPrivate::sendOperation(const QVariantMap &data)
 {
     QByteArray ba;
@@ -117,6 +139,15 @@ void UiProxyPrivate::sendOperation(const QVariantMap &data)
     m_ipc.write(ba);
 }
 
+void UiProxyPrivate::onDisconnected()
+{
+    Q_Q(UiProxy);
+
+    if (!m_finishedTimer.isActive()) {
+        Q_EMIT q->finished();
+    }
+}
+
 void UiProxyPrivate::onNewConnection()
 {
     Q_Q(UiProxy);
@@ -132,10 +163,12 @@ void UiProxyPrivate::onNewConnection()
 
     m_socket = socket;
     QObject::connect(socket, SIGNAL(disconnected()),
-                     q, SIGNAL(finished()));
+                     this, SLOT(onDisconnected()));
     m_ipc.setChannels(socket, socket);
     m_server.close(); // stop listening
 
+    setStatus(UiProxy::Ready);
+
     /* Execute any pending requests */
     QMapIterator<int, Request*> it(m_requests);
     while (it.hasNext()) {
@@ -178,7 +211,7 @@ bool UiProxyPrivate::setupSocket()
     QDir socketDir(runtimeDir + "/online-accounts-ui");
     if (!socketDir.exists()) socketDir.mkpath(".");
 
-    QString uniqueName = QString("ui-%1").arg(socketCounter++);
+    QString uniqueName = QString("ui-%1-%2").arg(socketCounter++).arg(m_providerId);
 
     /* If the file exists, it's a stale file: online-accounts-ui is a single
      * instance process, and the only one creating files in this directory. */
@@ -190,57 +223,112 @@ bool UiProxyPrivate::setupSocket()
     return m_server.listen(socketDir.filePath(uniqueName));
 }
 
-void UiProxyPrivate::setupPromptSession()
+bool UiProxyPrivate::setupPromptSession()
 {
     Q_Q(UiProxy);
 
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    if (!env.value("QT_QPA_PLATFORM").startsWith("ubuntu")) return;
+    if (!m_clientPid) return false;
 
     PromptSessionP session =
         MirHelper::instance()->createPromptSession(m_clientPid);
-    if (!session) return;
+    if (!session) return false;
 
     QString mirSocket = session->requestSocket();
     if (mirSocket.isEmpty()) {
-        return;
+        return false;
     }
 
     m_promptSession = session;
     QObject::connect(m_promptSession.data(), SIGNAL(finished()),
                      q, SIGNAL(finished()));
 
+    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
     env.insert("MIR_SOCKET", mirSocket);
     m_process.setProcessEnvironment(env);
+    return true;
 }
 
 bool UiProxyPrivate::init()
 {
-    if (Q_UNLIKELY(!setupSocket())) return false;
+    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+    if (env.value("QT_QPA_PLATFORM").startsWith("ubuntu")) {
+        if (!setupPromptSession()) {
+            qWarning() << "Couldn't setup prompt session";
+            return false;
+        }
+    }
+
+    /* We also create ~/cache/online-accounts-ui/, since the plugin might not
+     * have permissions to do that. */
+    QString userCacheDir =
+        QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation);
+    QDir cacheDir(userCacheDir + "/online-accounts-ui");
+    if (!cacheDir.exists()) cacheDir.mkpath(".");
+
+    return true;
+}
+
+QString UiProxyPrivate::findAppArmorProfile()
+{
+    if (Q_UNLIKELY(m_providerId.isEmpty())) return QString();
+
+    /* Load the provider XML file */
+    Accounts::Manager manager;
+    Accounts::Provider provider = manager.provider(m_providerId);
+    if (Q_UNLIKELY(!provider.isValid())) {
+        qWarning() << "Provider not found:" << m_providerId;
+        return QString();
+    }
+
+    const QDomDocument doc = provider.domDocument();
+    QDomElement root = doc.documentElement();
+    return root.firstChildElement("profile").text();
+}
+
+void UiProxyPrivate::startProcess()
+{
+    if (Q_UNLIKELY(!setupSocket())) {
+        qWarning() << "Couldn't setup IPC socket";
+        setStatus(UiProxy::Error);
+        return;
+    }
+    m_arguments.append("--socket");
+    m_arguments.append(m_server.fullServerName());
+
+    QString profile = findAppArmorProfile();
+    if (profile.isEmpty()) {
+        profile = "unconfined";
+    } else {
+        /* Set TMPDIR to a location which the confined process can actually
+         * use */
+        QString tmpdir =
+            QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) +
+            "/" + profile.split('_')[0];
+        QProcessEnvironment env = m_process.processEnvironment();
+        env.insert("TMPDIR", tmpdir);
+        m_process.setProcessEnvironment(env);
+    }
+
+    m_arguments.append("--profile");
+    m_arguments.append(profile);
 
     QString processName;
-    QStringList arguments;
     QString wrapper = QString::fromUtf8(qgetenv("OAU_WRAPPER"));
     QString accountsUi = QStringLiteral(INSTALL_BIN_DIR "/online-accounts-ui");
     if (wrapper.isEmpty()) {
         processName = accountsUi;
     } else {
         processName = wrapper;
-        arguments.append(accountsUi);
-    }
-    if (!m_promptSession) {
-        /* the first argument is required to be the desktop file */
-        arguments.append("--desktop_file_hint=/usr/share/applications/online-accounts-ui.desktop");
-    }
-    arguments.append("--socket");
-    arguments.append(m_server.fullServerName());
-
-    if (m_clientPid) {
-        setupPromptSession();
+        m_arguments.prepend(accountsUi);
     }
 
-    m_process.start(processName, arguments);
-    return m_process.waitForStarted();
+    setStatus(UiProxy::Loading);
+    m_process.start(processName, m_arguments);
+    if (Q_UNLIKELY(!m_process.waitForStarted())) {
+        qWarning() << "Couldn't start account plugin process";
+        setStatus(UiProxy::Error);
+        return;
+    }
 }
 
 void UiProxyPrivate::sendRequest(int requestId, Request *request)
@@ -285,6 +373,13 @@ UiProxy::UiProxy(pid_t clientPid, QObject *parent):
 
 UiProxy::~UiProxy()
 {
+    DEBUG();
+}
+
+UiProxy::Status UiProxy::status() const
+{
+    Q_D(const UiProxy);
+    return d->m_status;
 }
 
 bool UiProxy::init()
@@ -297,14 +392,19 @@ void UiProxy::handleRequest(Request *request)
 {
     Q_D(UiProxy);
 
+    if (d->m_providerId.isEmpty()) {
+        d->m_providerId = request->providerId();
+    }
     int requestId = d->m_nextRequestId++;
     d->m_requests.insert(requestId, request);
     QObject::connect(request, SIGNAL(completed()),
                      d, SLOT(onRequestCompleted()));
     request->setInProgress(true);
 
-    if (d->m_socket && d->m_socket->isValid()) {
+    if (d->m_status == UiProxy::Ready) {
         d->sendRequest(requestId, request);
+    } else if (d->m_status == UiProxy::Null) {
+        d->startProcess();
     }
 }
 
